# retrieve.py 
import os
import sys
import random
import logging
import time

logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(message)s')

import pylibm

sys.path.append('./esproto')
from esproto.common_pb2 import USER_SIGN_IN, USER_OBJ_GET, OK, Val, USER_SENS_VAL_GET
from esproto.userapi.system_pb2 import SignInReq, SignInResp
from esproto.userapi.object_pb2 import ObjGetReq, ObjGetResp, Obj
from esproto.userapi.sensor_pb2 import SensValGetReq, SensValGetResp

class ESSendError(Exception):
    pass

class IntializationError(Exception):
    pass

class ServiceLoginError(Exception):
    pass

class EventServerConnectionError(Exception):
    pass

class ESProtoError(Exception):
    pass

class ESConnection:
    def __init__(self):
        self.op_timeout = 15000
        self.login_request_id = -1
        self.request_objects_request_id = -1
        self.request_data_request_id = -1

        self.ctx = pylibm.mc_ctx()
        self.conn = pylibm.mc_conn(self.ctx)

        self.conn.set_msg_sent_cb(self.__on_message_sent)
        self.conn.set_conn_error_cb(self.__on_conn_error)
        self.conn.set_resp_recv_cb(self.__on_resp_recv)
        self.conn.set_conn_state_cb(self.__on_conn_state_change)
        self.conn.set_post_recv_cb(self.__on_post_recv)


    def initialize(self):
        self.__open_connection()


    def __open_connection(self):
        try:
            self.conn.open("127.0.0.1", 45001)
        except pylibm.error as err:
            logging.debug("Failed to open connection to ES server: {0}".format(str(err)))
            self.conn.abort()


    def __build_login_message(self):
        logging.info("Prepare singIn request")

        r = SignInReq()
        r.method = SignInReq.PLAIN
        r.user = "administratoruser"
        r.passwd = "aigsg@123"
        data = bytes(r.SerializeToString())
        # logging.debug(r.SerializeToString())

        m = pylibm.mc_msg(self.ctx, len(data))
        m.set_data(data)

        msg_type = USER_SIGN_IN
        msg_id = random.randrange(1000000)

        return m, msg_type, msg_id


    def __login(self):
        try:
            logging.info("Log in to ES server...")

            m, msg_type, self.login_request_id = self.__build_login_message() # store login request ID to identify it in callback
            self.conn.req_msg(m, msg_type, self.login_request_id, self.op_timeout)
            logging.info("Sing In request Sent")

        except pylibm.error as err:
            logging.debug("ParkingSensors Sync: {0}".format(str(err)))


    def __build_request_objects_message(self):
        logging.info("Prepare get objects request")

        r = ObjGetReq()
        r.flags.get_sens = True
        r.flags.get_actr = True
        r.flags.get_strm = False
        r.flags.get_obj_conf = True
        logging.info("Prepared get objects request")

        data = bytes(r.SerializeToString())
        m = pylibm.mc_msg(self.ctx, len(data))

        m.set_data(data)
        msg_type = USER_OBJ_GET
        msg_id = random.randrange(1000000)

        self.id_es_object = {}
        return m, msg_type, msg_id


    def __request_objects(self):
        try:
            logging.info("Requesting objects...")

            m, msg_type, self.request_objects_request_id = self.__build_request_objects_message() #store login request ID to identify it in callback
            self.conn.req_msg(m, msg_type, self.request_objects_request_id, self.op_timeout)
            logging.info("Request objects request Sent")

        except pylibm.error as err:
            logging.debug("ParkingSensors Sync: {0}".format(str(err)))


    def __parse_es_result(self, reply, Val, Obj):
        for obj in reply.obj:
            logging.debug(obj)

            if obj.cls == "sensor.Input":
                self.__request_sens_data(obj.id)


    def __process_objects(self, msg, result):
        logging.info("Processing objects")
        logging.info("Request objects result:  {0}".format(result))
        if result != OK:
            logging.error("Request for user objects failed")
            return

        reply = ObjGetResp()
        buf = msg.get_data()
        if buf is None:
            logging.warning("No es object found!")
            self.queue.put([])
            return

        reply.ParseFromString(buf)

        self.__parse_es_result(reply, Val, Obj)


    # Retrieve sens data
    def __build_request_data_message(self, objId):
        logging.info("Prepare get objects request")

        r = SensValGetReq()
        r.id.extend([ objId ])
        logging.info("Prepared get objects request")

        data = bytes(r.SerializeToString())
        m = pylibm.mc_msg(self.ctx, len(data))

        m.set_data(data)
        msg_type = USER_SENS_VAL_GET
        msg_id = random.randrange(1000000)

        return m, msg_type, msg_id


    def __request_sens_data(self, objId):
        try:
            logging.info("Requesting data...")

            m, msg_type, self.request_data_request_id = self.__build_request_data_message(objId)
            self.conn.req_msg(m, msg_type, self.request_data_request_id, self.op_timeout)
            logging.info("Request data Sent")

        except pylibm.error as err:
            logging.exception(err)


    # Callbacks
    def __on_resp_recv(self, conn, msg, type, result, req_id):
        logging.info("Response for Request with ID = {0} received.".format(req_id))
        try:
            if req_id == self.login_request_id:
                logging.info("Logged in.")
                self.logged_in = True
                self.__request_objects()

            if req_id == self.request_objects_request_id:
                logging.info("Received user objects")
                self.__process_objects(msg, result)

            if req_id == self.request_data_request_id:
                logging.info("Received sens data")
                assert result == OK

                reply = SensValGetResp()
                buf = msg.get_data()
                if buf is None:
                    logging.warning("No sens data found!")
                    return

                reply.ParseFromString(buf)
                logging.info(reply)

        except Exception as e:
            logging.exception(e)


    def __on_post_recv(self, conn, msg, type):
        logging.info("Post message received type = {0}, bytes = {1}".format(type, msg.get_size()))

        try:
            if type == USER_OBJ_CHG:
                logging.info('Objects changed')
                self.__request_objects()
        except Exception as e:
            logging.exception(e)


    def __on_message_sent(self, conn, req_id):
        if req_id == self.login_request_id:
            logging.info("Login request has been sent to ES server.")
        else:
            logging.info("Message with ID = {0} has been sent to ES".format(req_id))


    def __on_conn_error(self, c, err_code):
        logging.error("Connection error occured: {0}".format(self.print_error(err_code)))
        time.sleep(5)


    def __on_conn_state_change(self, conn, state_to, state_from):
        logging.debug("__on_conn_state_change")

        if state_to == pylibm.MC_CONN_STATE_CLOSED:
            logging.info("Connection to ES server is Closed")
            time.sleep(5)
            self.initialize()
        elif state_to == pylibm.MC_CONN_STATE_OPEN:
            logging.info("Connection to ES server is Opened")
            self.__login()
        elif state_to == pylibm.MC_CONN_STATE_OPENING:
            logging.info("Opening connection to ES server...")
        elif state_to == pylibm.MC_CONN_STATE_CLOSING:
            logging.info("Closing connection to ES server...")


connection = ESConnection()
connection.initialize()

try:
    while 1:
        time.sleep(1)
except KeyboardInterrupt:
    pass
finally:
    del connection